import _extends from "@babel/runtime-corejs3/helpers/extends";
import { useInstance, createRef, onMounted, onBeforeUnmount } from 'intact-vue-next';
import { useState } from '../../hooks/useState';
import { useResizeObserver } from '../../hooks/useResizeObserver';
export function useEditor() {
  var instance = useInstance();
  var elementRef = createRef();
  var loading = useState(true);
  var codeEditor = null;
  onMounted(function () {
    import('monaco-editor').then(function (monaco) {
      if (instance.$unmounted) return;
      console.log(monaco);

      var _instance$get = instance.get(),
          value = _instance$get.value,
          language = _instance$get.language,
          theme = _instance$get.theme,
          readOnly = _instance$get.readOnly,
          options = _instance$get.options;

      var Editor = monaco.editor;
      codeEditor = Editor.create(elementRef.value, _extends({
        value: value,
        language: language,
        theme: theme,
        readOnly: readOnly
      }, options));
      watch(Editor, codeEditor);
      loading.set(false);
      instance.trigger('ready', codeEditor, Editor);
    });
  });
  onBeforeUnmount(function () {
    if (codeEditor) {
      codeEditor.dispose();
      codeEditor = null;
    }
  });
  useResizeObserver(elementRef, function () {
    if (codeEditor) {
      codeEditor.layout();
    }
  });

  function watch(Editor, codeEditor) {
    instance.on('$receive:language', function (v) {
      Editor.setModelLanguage(codeEditor.getModel(), v);
    });
    instance.on('$receive:theme', function (v) {
      Editor.setTheme(v);
    });
    instance.on('$receive:options', function (v) {
      codeEditor.updateOptions(v);
    });
    instance.on('$receive:value', function (v) {
      codeEditor.setValue(v);
    });
    instance.on('$receive:readOnly', function (v) {
      codeEditor.updateOptions({
        readOnly: v
      });
    });
    codeEditor.onDidChangeModelContent(function (e) {
      instance.set('value', codeEditor.getValue());
    });
  }

  return {
    elementRef: elementRef,
    loading: loading
  };
}