/**
 * The base hook for handling value of Datepicker and Timepicker
 */
import { TypeDefs } from 'intact-vue-next';
import { State } from '../../hooks/useState';
import { Dayjs, OpUnitType } from './dayjs';
import type { useFormats } from './useFormats';
import type { useDisabled } from './useDisabled';
import { PanelFlags, usePanel } from './usePanel';
import { BaseSelect, BaseSelectProps, BaseSelectEvents, BaseSelectBlocks } from '../select/base';
export declare type StateValueRange = [Dayjs, Dayjs?];
export declare type StateValueItem = Dayjs | StateValueRange;
export declare type StateValue = StateValueItem[];
export declare type StringValue = string | string[] | [string, string] | [string, string][];
export declare type DayjsValueRange = [Dayjs, Dayjs];
export declare type DayjsValueItem = Dayjs | DayjsValueRange;
export declare type DayjsValue = DayjsValueItem[];
export interface BasePickerProps<Value> extends BaseSelectProps {
    value?: Value | Value[] | [Value, Value] | [Value, Value][] | null;
    range?: boolean;
    format?: string;
    valueFormat?: string;
    showFormat?: string;
    min?: Value;
    max?: Value;
    disabledDate?: (v: Value) => boolean;
}
export interface BasePickerEvents extends BaseSelectEvents {
}
export interface BasePickerBlocks extends BaseSelectBlocks {
}
export declare type Value = string | Date | number | Dayjs;
export declare abstract class BasePicker<T extends BasePickerProps<any> = BasePickerProps<any>, E extends BasePickerEvents = BasePickerEvents, B extends BasePickerBlocks = BasePickerBlocks> extends BaseSelect<T, E, B> {
    static typeDefs: Required<TypeDefs<BasePickerProps<Value>>>;
    static defaults: () => Partial<BasePickerProps<any>>;
    abstract value: ReturnType<typeof useValue>;
    abstract formats: ReturnType<typeof useFormats>;
    resetKeywords(keywords: State<string>): void;
}
export declare function useValue({ createDateByValueFormat, createDateByShowFormat, getShowString, getValueString, }: ReturnType<typeof useFormats>, { isDisabled }: ReturnType<typeof useDisabled>, panel: ReturnType<typeof usePanel>, shouldUpdateValue: (v: StateValueItem) => boolean, updateValueOnInput: (v: DayjsValueItem) => void, getEqualType: () => OpUnitType): {
    value: State<StateValue>;
    format: () => string | string[];
    onConfirm: () => void;
    onChangeTime: (date: Dayjs, flag: PanelFlags) => void;
    getTimeValue: (flag: PanelFlags) => Dayjs | null | undefined;
    setValue: (v: StateValueItem, fromInput: boolean) => void;
    convertToDayjs: (v: BasePickerProps<Value>['value']) => DayjsValue;
    getDayjsValue: () => DayjsValue;
};
