import { TypeDefs } from 'intact-vue-next';
import dayjs from './dayjs';
import { State } from '../../hooks/useState';
import { Shortcut } from './shortcuts';
import { BasePicker, BasePickerProps, BasePickerEvents, BasePickerBlocks, Value } from './basepicker';
export * as shortcuts from './shortcuts';
export interface DatepickerProps extends BasePickerProps<Value> {
    type?: 'date' | 'datetime' | 'year' | 'month';
    shortcuts?: Shortcut[];
}
export interface DatepickerEvents extends BasePickerEvents {
}
export interface DatepickerBlocks extends BasePickerBlocks {
}
export declare class Datepicker extends BasePicker<DatepickerProps, DatepickerEvents, DatepickerBlocks> {
    static template: import("intact").Template<any>;
    static typeDefs: Required<TypeDefs<DatepickerProps>>;
    static defaults: () => Partial<DatepickerProps>;
    formats: {
        getValueFormat: () => string;
        getShowFormat: () => string;
        createDateByValueFormat: (value: Value) => dayjs.Dayjs;
        createDateByShowFormat: (value: string) => dayjs.Dayjs;
        getShowString: (value: dayjs.Dayjs) => string;
        getValueString: (value: dayjs.Dayjs) => string;
    };
    disabled: {
        isDisabled: (value: dayjs.Dayjs, type?: "second" | "month" | "year" | "date") => boolean;
        isDisabledTime: (value: dayjs.Dayjs, flag: import("./usePanel").PanelFlags) => boolean;
        isDisabledConfirm: () => boolean;
        maxDate: State<dayjs.Dayjs | null>;
        minDate: State<dayjs.Dayjs | null>;
    };
    panel: {
        startPanel: State<import("./usePanel").PanelTypes>;
        endPanel: State<import("./usePanel").PanelTypes>;
        changePanel: (type: import("./usePanel").PanelTypes, flag?: import("./usePanel").PanelFlags) => void;
        getPanel: (flag: import("./usePanel").PanelFlags) => State<import("./usePanel").PanelTypes>;
        reset: () => void;
        startRef: import("intact").RefObject<import("./calendar").DatepickerCalendar>;
        endRef: import("intact").RefObject<import("./calendar").DatepickerCalendar>;
    };
    focusDate: {
        focusDate: State<dayjs.Dayjs | null>;
        reset: () => void;
    };
    value: {
        format: () => string | string[];
        onConfirm: () => void;
        onChangeTime: (date: dayjs.Dayjs, flag: import("./usePanel").PanelFlags) => void;
        getTimeValue: (flag: import("./usePanel").PanelFlags) => dayjs.Dayjs | null | undefined;
        convertToDayjs: (v: Value | Value[] | [Value, Value] | [Value, Value][] | null | undefined) => import("./basepicker").DayjsValue;
        getDayjsValue: () => import("./basepicker").DayjsValue;
        value: State<import("./basepicker").StateValue>;
        setValue: (v: import("./basepicker").StateValueItem, fromInput: boolean) => void;
        onChangeDate: (v: dayjs.Dayjs, flag: import("./usePanel").PanelFlags) => void;
    };
    init(): void;
    protected getPlaceholder(): string | number | boolean | import("misstime/dist/utils/types").VNode<any> | import("intact").Children[];
    protected getLabel(): string | string[];
    protected clear(e: MouseEvent): void;
    private setByShortcut;
}
