import _inheritsLoose from "@babel/runtime-corejs3/helpers/inheritsLoose";
import _concatInstanceProperty from "@babel/runtime-corejs3/core-js/instance/concat";
import _Map from "@babel/runtime-corejs3/core-js/map";
import _Set from "@babel/runtime-corejs3/core-js/set";
import { __decorate } from "tslib";
import { Component, createRef, provide } from 'intact-vue-next';
import template from './diagram.vdt';
import { createGraph } from './graph';
import mx from './mxgraph';
import { bind } from '../utils';
import { DIAGRAM } from './constants';
import { error, isNullOrUndefined } from 'intact-shared';
var mxRubberband = mx.mxRubberband,
    mxEvent = mx.mxEvent;
export var Diagram = /*#__PURE__*/function (_Component) {
  _inheritsLoose(Diagram, _Component);

  function Diagram() {
    var _context;

    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _Component.call.apply(_Component, _concatInstanceProperty(_context = [this]).call(_context, args)) || this;
    _this.graph = void 0;
    _this.cell = void 0;
    _this.shapes = new _Map();
    _this.lines = new _Set();
    _this.layouts = new _Set();
    _this.canvasRef = createRef();
    _this.isCellSelectable = void 0;
    _this.isCellRotatable = void 0;
    _this.isCellMovable = void 0;
    _this.isCellResizable = void 0;
    _this.isCellEditable = void 0;
    _this.isCellConnectable = void 0;
    return _this;
  }

  var _proto = Diagram.prototype;

  _proto.init = function init() {
    provide(DIAGRAM, this);
  };

  _proto.mounted = function mounted() {
    var _this2 = this;

    var graph = this.graph = createGraph(this.canvasRef.value);
    this.isCellSelectable = graph.isCellSelectable;
    this.isCellRotatable = graph.isCellRotatable;
    this.isCellMovable = graph.isCellMovable;
    this.isCellResizable = graph.isCellResizable;
    this.isCellEditable = graph.isCellEditable;
    this.isCellConnectable = graph.isCellConnectable; // add label changed event

    graph.addListener(mxEvent.LABEL_CHANGED, function (graph, evt) {
      var _evt$properties = evt.properties,
          cell = _evt$properties.cell,
          old = _evt$properties.old,
          value = _evt$properties.value;

      _this2.trigger('labelChanged', cell, value, old);
    });
    graph.getSelectionModel().addListener(mxEvent.CHANGE, this.onSelectionChange);
    this.initGraph();
    this.cell = graph.getDefaultParent();
    this.draw();
  };

  _proto.updated = function updated() {
    this.draw();
  };

  _proto.beforeUnmount = function beforeUnmount() {
    this.canvasRef.value.removeAttribute('style');
    this.graph.destroy();
  };

  _proto.addShape = function addShape(shape) {
    var _shape$get = shape.get(),
        key = _shape$get.key;

    var shapes = this.shapes;

    if (!isNullOrUndefined(key)) {
      if (shapes.get(key)) {
        error("The key: [" + key + "] of [" + shape.constructor.displayName + "] is duplicate.");
      } else {
        shapes.set(key, shape);
      }
    } else {
      shapes.set(shape.uniqueId, shape);
    }
  };

  _proto.deleteShape = function deleteShape(shape) {
    var _shape$get2 = shape.get(),
        key = _shape$get2.key;

    this.shapes.delete(isNullOrUndefined(key) ? shape.uniqueId : key);
  };

  _proto.addLine = function addLine(line) {
    this.lines.add(line);
  };

  _proto.deleteLine = function deleteLine(line) {
    this.lines.delete(line);
  };

  _proto.addLayout = function addLayout(layout) {
    this.layouts.add(layout);
  };

  _proto.deleteLayout = function deleteLayout(layout) {
    this.layouts.delete(layout);
  };

  _proto.onSelectionChange = function onSelectionChange() {
    this.trigger('selectionChanged', this.graph.getSelectionCells());
  };

  _proto.initGraph = function initGraph() {
    var _this3 = this;

    var graph = this.graph;
    graph.setEnabled(true);
    ['selectable', 'movable', 'resizable', 'rotatable', 'editable', 'connectable'].forEach(function (item) {
      var name = "isCell" + item[0].toUpperCase() + item.substring(1);

      graph[name] = function (cell) {
        return cell.instance && cell.instance.get(item) && _this3[name].call(graph, cell);
      };
    }); // this.rubberband.setEnabled(selectable);

    graph.resizeContainer = true; // graph.gridSize = 0;
  };

  _proto.draw = function draw() {
    var graph = this.graph;
    var model = graph.model;
    model.beginUpdate();

    try {
      this.shapes.forEach(function (shape) {
        shape.draw();
      }); // we render lines after all vertexes has rendered

      this.lines.forEach(function (line) {
        line.draw();
      }); // render layout

      this.layouts.forEach(function (layout) {
        layout.draw();
      });
    } finally {
      model.endUpdate();
    }
  };

  return Diagram;
}(Component);
Diagram.template = template;

__decorate([bind], Diagram.prototype, "onSelectionChange", null);