import _inheritsLoose from "@babel/runtime-corejs3/helpers/inheritsLoose";
import _concatInstanceProperty from "@babel/runtime-corejs3/core-js/instance/concat";
import _Set from "@babel/runtime-corejs3/core-js/set";
import _mapInstanceProperty from "@babel/runtime-corejs3/core-js/instance/map";
import _Array$from from "@babel/runtime-corejs3/core-js/array/from";
import { Component, createVNode as h, inject, provide, isComponentClass } from 'intact-vue-next';
import { eachChildren } from '../../utils';
import { DBase } from '../shapes/base';
import { DIAGRAM, SHAPE, LAYOUT } from '../constants';
import { isStringOrNumber, error } from 'intact-shared';
var typeDefs = {
  left: [String, Number],
  top: [String, Number]
};

var defaults = function defaults() {
  return {
    left: 0,
    top: 0
  };
};

export var DLayout = /*#__PURE__*/function (_Component) {
  _inheritsLoose(DLayout, _Component);

  function DLayout() {
    var _context;

    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _Component.call.apply(_Component, _concatInstanceProperty(_context = [this]).call(_context, args)) || this;
    _this.diagram = inject(DIAGRAM);
    _this.shapes = new _Set();
    _this.shape = inject(SHAPE, null);
    return _this;
  }

  var _proto = DLayout.prototype;

  _proto.init = function init() {
    provide(LAYOUT, this);
  };

  _proto.mounted = function mounted() {
    this.diagram.addLayout(this);
  };

  _proto.beforeUnmount = function beforeUnmount() {
    this.diagram.deleteLayout(this);
  };

  _proto.addShape = function addShape(shape) {
    this.shapes.add(shape);
  };

  _proto.deleteShape = function deleteShape(shape) {
    this.shapes.delete(shape);
  };

  _proto.draw = function draw() {
    var _context2;

    var _this$get = this.get(),
        left = _this$get.left,
        top = _this$get.top;

    var diagram = this.diagram;
    var graph = diagram.graph;
    var layout = this.getLayout(graph); // Allows the layout to move cells even though cells
    // aren't movable in the graph

    layout.isVertexMovable = function () {
      return true;
    };

    var cells = _mapInstanceProperty(_context2 = _Array$from(this.shapes)).call(_context2, function (shape) {
      return shape.cell;
    });

    var parent = this.shape ? this.shape : this.diagram;
    this.execute(layout, cells, parent, graph); // move cells totally

    graph.moveCells(cells, +left, +top);
  };

  _proto.execute = function execute(layout, cells, parent, graph) {
    layout.execute(parent.cell, cells);
  };

  return DLayout;
}(Component);

DLayout.template = function () {
  if (process.env.NODE_ENV !== 'production') {
    eachChildren(this.get('children'), function (vNode) {
      if (isStringOrNumber(vNode)) return;
      if (isComponentClass(vNode) && vNode.tag.prototype instanceof DBase) return;
      error("DLayout can only contain DShape or DLine, but " + vNode.tag + " found.");
    });
  }

  var constructor = this.constructor;
  return h(constructor.displayName, null, this.get('children'));
};

DLayout.typeDefs = typeDefs;
DLayout.defaults = defaults;