import { Component, TypeDefs, VNodeComponentClass } from 'intact-vue-next';
import type { Diagram } from '../diagram';
export declare type Cell = {
    data?: any;
    instance?: DBase;
    vertex?: boolean;
    edge?: boolean;
    getGeometry: () => any;
    setTerminal: (...args: any[]) => void;
};
export interface DBaseProps {
    strokeStyle?: 'solid' | 'dashed' | 'dotted';
    label?: string | number;
    style?: Record<string, string | number>;
    data?: any;
}
export declare abstract class DBase<P extends DBaseProps = DBaseProps> extends Component<P> {
    static typeDefs: Required<TypeDefs<DBaseProps>>;
    static defaults: () => Partial<DBaseProps>;
    uniqueId: number;
    cell: Cell | null;
    protected diagram: Diagram;
    private hasDrew;
    private layout;
    mounted(): void;
    beforeUpdate(lastVNode: VNodeComponentClass<DBase>, nextVNode: VNodeComponentClass<DBase>): void;
    beforeUnmount(): void;
    draw(): void;
    protected abstract startDraw(): void;
    protected abstract addToDigram(): void;
    protected abstract deleteFromDiagram(): void;
    protected abstract getStylesheet(): string;
    protected abstract getGeometry(): any;
    protected updateProps(model: any, lastProps: DBaseProps, nextProps: DBaseProps): void;
    protected createCell(): any;
    protected getValue(): string | null | Element;
    protected setStyle(): void;
    protected genStyles(): {
        dashed?: number | null | undefined;
        dashPattern?: string | null | undefined;
    };
}
