import _extends from "@babel/runtime-corejs3/helpers/extends";
import _inheritsLoose from "@babel/runtime-corejs3/helpers/inheritsLoose";
import _concatInstanceProperty from "@babel/runtime-corejs3/core-js/instance/concat";
import { Component, inject } from 'intact-vue-next';
import mx from '../mxgraph';
import { isNullOrUndefined, EMPTY_OBJ } from 'intact-shared';
import { DIAGRAM, LAYOUT } from '../constants';
var mxCell = mx.mxCell,
    mxGeometry = mx.mxGeometry;
var typeDefs = {
  strokeStyle: ['solid', 'dashed', 'dotted'],
  label: [String, Number],
  style: Object,
  data: null
};

var defaults = function defaults() {
  return {
    strokeStyle: 'solid'
  };
};

var uniqueId = 0;
export var DBase = /*#__PURE__*/function (_Component) {
  _inheritsLoose(DBase, _Component);

  function DBase() {
    var _context;

    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _Component.call.apply(_Component, _concatInstanceProperty(_context = [this]).call(_context, args)) || this;
    _this.uniqueId = uniqueId++;
    _this.cell = null;
    _this.diagram = inject(DIAGRAM);
    _this.hasDrew = false;
    _this.layout = inject(LAYOUT, null);
    return _this;
  }

  var _proto = DBase.prototype;

  _proto.mounted = function mounted() {
    var _this$get = this.get(),
        data = _this$get.data;

    var cell = this.cell = this.createCell(); // add data to the cell, so that we can get it when select the cell

    cell.data = data;
    cell.instance = this;
    this.addToDigram();

    if (this.layout) {
      this.layout.addShape(this);
    }
  };

  _proto.beforeUpdate = function beforeUpdate(lastVNode, nextVNode) {
    var lastProps = lastVNode.props || EMPTY_OBJ;
    var nextProps = nextVNode.props || EMPTY_OBJ;

    if (lastProps !== nextProps) {
      var graph = this.diagram.graph;
      var model = graph.getModel();
      model.beginUpdate();
      this.updateProps(model, lastProps, nextProps);
      model.endUpdate();
    }
  };

  _proto.beforeUnmount = function beforeUnmount() {
    var graph = this.diagram.graph;
    this.deleteFromDiagram();
    graph.removeCells([this.cell], false);

    if (this.layout) {
      this.layout.deleteShape(this);
    }
  };

  _proto.draw = function draw() {
    if (this.hasDrew) return;
    this.startDraw();
    this.hasDrew = true;
  };

  _proto.updateProps = function updateProps(model, lastProps, nextProps) {
    var cell = this.cell;

    if (lastProps.label !== nextProps.label) {
      // update value
      model.setValue(cell, this.getValue());
    } // update style


    model.setStyle(cell, this.getStylesheet());
    this.setStyle(); // update data

    cell.data = nextProps.data;
  };

  _proto.createCell = function createCell() {
    return new mxCell(this.getValue(), this.getGeometry(), this.getStylesheet());
  };

  _proto.getValue = function getValue() {
    var _this$get2 = this.get(),
        label = _this$get2.label;

    return isNullOrUndefined(label) ? null : String(label);
  };

  _proto.setStyle = function setStyle() {
    var graph = this.diagram.graph;
    var cells = [this.cell];

    var styles = _extends({}, this.get('style'), this.genStyles());

    for (var key in styles) {
      graph.setCellStyles(key, styles[key], cells);
    }
  };

  _proto.genStyles = function genStyles() {
    var styles = {};

    switch (this.get('strokeStyle')) {
      case 'dashed':
        styles.dashed = 1;
        styles.dashPattern = null;
        break;

      case 'dotted':
        styles.dashed = 1;
        styles.dashPattern = '1 2';
        break;

      case 'solid':
        styles.dashed = null;
        styles.dashPattern = null;
        break;
    }

    return styles;
  };

  return DBase;
}(Component);
DBase.typeDefs = typeDefs;
DBase.defaults = defaults;