import _construct from "@babel/runtime-corejs3/helpers/construct";
import _inheritsLoose from "@babel/runtime-corejs3/helpers/inheritsLoose";
import _extends from "@babel/runtime-corejs3/helpers/extends";
import _findInstanceProperty from "@babel/runtime-corejs3/core-js/instance/find";
import { createVNode as h } from 'intact-vue-next';
import mx from '../mxgraph';
import { DBase } from './base';
import { isNullOrUndefined } from 'intact-shared';
var mxCell = mx.mxCell,
    mxGeometry = mx.mxGeometry,
    mxPoint = mx.mxPoint;
var arrows = ['none', 'classic', 'classicThin', 'block', 'blockThin', 'open', 'openThin', 'oval', 'diamond', 'diamondThin'];

var typeDefs = _extends({}, DBase.typeDefs, {
  strokeStyle: ['solid', 'dashed', 'dotted'],
  type: ['rounded', 'sharp', 'curved', 'straight'],
  startArrow: arrows,
  endArrow: arrows,
  startPoint: Array,
  endPoint: Array,
  exit: Array,
  entry: Array,
  from: [String, Number],
  to: [String, Number]
});

var defaults = function defaults() {
  return _extends({}, DBase.defaults(), {
    startArrow: 'none',
    endArrow: 'classic',
    startPoint: [0, 0],
    endPoint: [50, 0],
    type: 'straight'
  });
};

export var DLine = /*#__PURE__*/function (_DBase) {
  _inheritsLoose(DLine, _DBase);

  function DLine() {
    return _DBase.apply(this, arguments) || this;
  }

  var _proto = DLine.prototype;

  _proto.startDraw = function startDraw() {
    var graph = this.diagram.graph;
    var cell = this.cell;
    cell.edge = true;
    this.setStyle();
    this.setTerminal();
    graph.addCell(cell);
  };

  _proto.setTerminal = function setTerminal() {
    var _this$get = this.get(),
        from = _this$get.from,
        to = _this$get.to,
        startPoint = _this$get.startPoint,
        endPoint = _this$get.endPoint;

    var diagram = this.diagram;
    var graph = diagram.graph;
    var shapes = diagram.shapes;
    var cell = this.cell;
    var geo = cell.getGeometry();
    var tmp;

    if (isNullOrUndefined(from) || !(tmp = shapes.get(from))) {
      cell.setTerminal(null, true);
      geo.setTerminalPoint(_construct(mxPoint, startPoint), true);
    } else {
      cell.setTerminal(tmp.cell, true);
    }

    if (isNullOrUndefined(to) || !(tmp = shapes.get(to))) {
      cell.setTerminal(null, false);
      geo.setTerminalPoint(_construct(mxPoint, endPoint), false);
    } else {
      cell.setTerminal(tmp.cell, false);
    }
  };

  _proto.addToDigram = function addToDigram() {
    this.diagram.addLine(this);
  };

  _proto.deleteFromDiagram = function deleteFromDiagram() {
    this.diagram.deleteLine(this);
  };

  _proto.getStylesheet = function getStylesheet() {
    var _this$get2 = this.get(),
        startArrow = _this$get2.startArrow,
        endArrow = _this$get2.endArrow;

    return "startArrow=" + startArrow + ";endArrow=" + endArrow + ";html=1;fontSize=12;";
  };

  _proto.getGeometry = function getGeometry() {
    var geo = new mxGeometry(0, 0, 0, 0);
    geo.relative = true;
    return geo;
  };

  _proto.genStyles = function genStyles() {
    return _extends({}, this.genStylesForExitAndEntry(), this.genStylesForType(), _DBase.prototype.genStyles.call(this));
  };

  _proto.updateProps = function updateProps(model, lastProps, nextProps) {
    var _context;

    _DBase.prototype.updateProps.call(this, model, lastProps, nextProps);

    if (_findInstanceProperty(_context = ['from', 'to', 'startPoint', 'endPoint']).call(_context, function (key) {
      return lastProps[key] !== nextProps[key];
    })) {
      // update from and to
      this.setTerminal();
    }
  };

  _proto.genStylesForExitAndEntry = function genStylesForExitAndEntry() {
    var styles = {};

    var _this$get3 = this.get(),
        from = _this$get3.from,
        exit = _this$get3.exit,
        to = _this$get3.to,
        entry = _this$get3.entry;

    if (!isNullOrUndefined(from) && exit) {
      styles.exitX = exit[0];
      styles.exitY = exit[1];
      styles.perimeter = 0;
    }

    if (!isNullOrUndefined(to) && entry) {
      styles.entryX = entry[0];
      styles.entryY = entry[1];
      styles.perimeter = 0;
    }

    return styles;
  };

  _proto.genStylesForType = function genStylesForType() {
    var styles = {};

    var _this$get4 = this.get(),
        type = _this$get4.type;

    if (type !== 'straight') {
      styles.edgeStyle = 'orthogonalEdgeStyle';
    }

    switch (type) {
      case 'rounded':
        styles.rounded = 1;
        styles.curved = null;
        break;

      case 'curved':
        styles.rounded = 0;
        styles.curved = 1;
        break;

      case 'sharp':
        styles.rounded = 0;
        styles.curved = null;
        break;

      default:
        styles.edgeStyle = 'straight';
        styles.curved = null;
        styles.noEdgeStyle = null;
        break;
    }

    return styles;
  };

  return DLine;
}(DBase);

DLine.template = function () {
  return h('dline', null, this.get('children'));
};

DLine.typeDefs = typeDefs;
DLine.defaults = defaults;