import _inheritsLoose from "@babel/runtime-corejs3/helpers/inheritsLoose";
import _extends from "@babel/runtime-corejs3/helpers/extends";
import _concatInstanceProperty from "@babel/runtime-corejs3/core-js/instance/concat";
import { createVNode as h, createRef, isComponentClass, provide, inject } from 'intact-vue-next';
import { mapChildren } from '../../utils';
import mx from '../mxgraph';
import { isStringOrNumber, isNullOrUndefined } from 'intact-shared';
import { SHAPE } from '../constants';
import { DBase } from './base';
var mxCell = mx.mxCell,
    mxGeometry = mx.mxGeometry;
var strOrNum = [String, Number];

var typeDefs = _extends({}, DBase.typeDefs, {
  left: strOrNum,
  top: strOrNum,
  width: strOrNum,
  height: strOrNum,
  movable: Boolean,
  selectable: Boolean,
  resizable: Boolean,
  rotatable: Boolean,
  editable: Boolean,
  connectable: Boolean
});

var defaults = function defaults() {
  return _extends({}, DBase.defaults(), {
    left: 0,
    top: 0,
    width: 0,
    height: 0
  });
};

export var DShape = /*#__PURE__*/function (_DBase) {
  _inheritsLoose(DShape, _DBase);

  function DShape() {
    var _context;

    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _DBase.call.apply(_DBase, _concatInstanceProperty(_context = [this]).call(_context, args)) || this;
    _this.wrapperRef = createRef();
    _this.hasElement = false;
    _this.children = null;
    _this.parent = inject(SHAPE, null);
    return _this;
  }

  var _proto = DShape.prototype;

  _proto.init = function init() {
    var _this2 = this;

    // shape can nest
    provide(SHAPE, this);
    this.on('$receive:children', function (children) {
      _this2.children = mapChildren(children, function (vNode) {
        if (isStringOrNumber(vNode)) return vNode;

        if (isComponentClass(vNode)) {
          var proto = vNode.tag.prototype;

          if (proto instanceof DShape) {
            return vNode;
          }
        }

        _this2.hasElement = true;
        return vNode;
      });
    });
  };

  _proto.updated = function updated() {
    var model = this.diagram.graph.model;
    var geo = this.getGeometry();
    model.setGeometry(this.cell, geo);
  };

  _proto.beforeUnmount = function beforeUnmount() {
    var graph = this.diagram.graph;
    this.deleteFromDiagram();
    graph.removeCells([this.cell], false);
  };

  _proto.startDraw = function startDraw() {
    var diagram = this.diagram;
    var cell = this.cell;
    var parentCell = this.parent ? this.parent.cell : diagram.cell;
    cell.vertex = true;
    this.setStyle();
    diagram.graph.addCell(cell, parentCell);
  };

  _proto.addToDigram = function addToDigram() {
    this.diagram.addShape(this);
  };

  _proto.deleteFromDiagram = function deleteFromDiagram() {
    this.diagram.addShape(this);
  };

  _proto.getGeometry = function getGeometry() {
    var _this$get = this.get(),
        width = _this$get.width,
        height = _this$get.height,
        top = _this$get.top,
        left = _this$get.left;

    return new mxGeometry(+left, +top, +width, +height);
  };

  _proto.getValue = function getValue() {
    var _this$get2 = this.get(),
        label = _this$get2.label;

    return isNullOrUndefined(label) ? this.hasElement && this.wrapperRef.value || null : String(label);
  };

  _proto.setStyle = function setStyle() {
    // if hasElement, disable edit
    this.diagram.graph.setCellStyles('editable', this.hasElement ? 0 : 1, [this.cell]);

    _DBase.prototype.setStyle.call(this);
  };

  return DShape;
}(DBase);

DShape.template = function () {
  var children = this.children;

  if (children && children.length) {
    children = h('div', {
      ref: this.wrapperRef
    }, children);
  }

  var Ctor = this.constructor;
  return h(Ctor.displayName || Ctor.name, null, children);
};

DShape.typeDefs = typeDefs;
DShape.defaults = defaults;