import _mapInstanceProperty from "@babel/runtime-corejs3/core-js/instance/map";
import { css } from '@emotion/css';
import { theme } from '../../styles/theme';
import { deepDefaults } from '../../styles/utils';
import '../../styles/global';

var _deepDefaults = deepDefaults(theme, {
  dialog: {
    width: '800px',
    bgColor: '#fff',

    get borderRadius() {
      return theme.borderRadius;
    },

    padding: '0',
    margin: '0 20px',

    get color() {
      return theme.color.text;
    },

    get fontSize() {
      return theme.fontSize;
    },

    get boxShadow() {
      return theme.boxShadow;
    },

    // header
    header: {
      fontSize: '16px',

      get color() {
        return theme.color.title;
      },

      height: '61px',
      border: "1px solid #eaeaea",
      closeTop: '16px',
      closeRight: '-9px',
      closeIconFontSize: '44px'
    },
    // body
    body: {
      padding: "20px"
    },
    // footer
    footer: {
      padding: "20px 0",
      btnGap: "12px",
      border: "1px solid #eaeaea"
    },
    // transition
    transform: 'translateY(-10px) scale(1.05)',
    // size
    largeWidth: '1000px',
    smallWidth: '600px',
    miniWidth: '400px',
    // alert dialog
    alert: {
      padding: "0 20px",
      bodyMarginTop: "-25px",
      tipIconMarginBottom: '10px',
      tipIconFontSize: '37px',
      tipIconLineHeight: '37px',
      // with title
      titleTipIconFontSize: '37px',
      titleFontSize: '14px',
      wrapperPaddingLeft: '8px',
      titleBodyMarginTop: '-36px'
    }
  }
}),
    dialog = _deepDefaults.dialog;

export function makeDialogStyles() {
  var _context;

  return /*#__PURE__*/css("position:absolute;width:", dialog.width, ";border-radius:", dialog.borderRadius, ";background:", dialog.bgColor, ";font-size:", dialog.fontSize, ";color:", dialog.color, ";box-shadow:", dialog.boxShadow, ";max-width:100%;z-index:", theme.maxZIndex, ";&.k-dragging{cursor:move;user-select:none;}&.transition-enter-active,&.transition-leave-active,&.transition-appear-active{transition:all ", theme.transition, ";}&.transition-enter-from,&.transition-leave-to,&.transition-appear-from{transform:", dialog.transform, ";opacity:0;}", _mapInstanceProperty(_context = ['large', 'small', 'mini']).call(_context, function (size) {
    return /*#__PURE__*/css("&.k-", size, "{width:", dialog[size + "Width"], ";}");
  }), ".k-dialog-header{padding:0 ", dialog.padding, ";margin:", dialog.margin, ";border-bottom:", dialog.header.border, ";height:", dialog.header.height, ";line-height:", dialog.header.height, ";font-size:", dialog.header.fontSize, ";color:", dialog.header.color, ";position:relative;}.k-dialog-title{display:inline-block;}.k-dialog-close{position:absolute;right:", dialog.header.closeRight, ";top:", dialog.header.closeTop, ";.k-icon{font-size:", dialog.header.closeIconFontSize, ";}}.k-dialog-body{padding:", dialog.body.padding, ";}.k-dialog-footer{text-align:right;padding:", dialog.footer.padding, ";margin:", dialog.margin, ";border-top:", dialog.footer.border, ";.k-btn{margin-left:", dialog.footer.btnGap, ";}}");
}
export function makeWrapperStyles() {
  // k-fade-leave-active will add position absolute to the styles
  // so we must set fixed with important to .k-dialog-overlay
  return /*#__PURE__*/css("position:absolute;z-index:", theme.maxZIndex, ";top:0;left:0;width:100%;.k-dialog-overlay{position:fixed!important;top:0;left:0;width:100%;height:100%;background:rgba(0, 0, 0, .5);}");
}
export function makeAlertStyles() {
  var _context2;

  return /*#__PURE__*/css("&.k-alert-dialog{.k-dialog-body{margin-top:", dialog.alert.bodyMarginTop, ";padding:", dialog.alert.padding, ";text-align:center;position:relative;z-index:1;}.k-dialog-header,.k-dialog-footer{border:none;}.k-dialog-tip-icon{margin-bottom:", dialog.alert.tipIconMarginBottom, ";.k-icon{font-size:", dialog.alert.tipIconFontSize, ";line-height:", dialog.alert.tipIconLineHeight, ";}}", _mapInstanceProperty(_context2 = ['success', 'warning', 'error', 'confirm']).call(_context2, function (type) {
    var color = type === 'error' ? theme.color.danger : type === 'confirm' ? theme.color.primary : theme.color[type];
    return /*#__PURE__*/css("&.k-", type, "{.k-dialog-tip-icon{color:", color, ";}}");
  }), " &:not(.k-confirm){.k-dialog-cancel{display:none;}}&.k-with-title{.k-dialog-body{margin-top:", dialog.alert.titleBodyMarginTop, ";text-align:left;}.k-dialog-tip-icon{float:left;.k-icon{font-size:", dialog.alert.titleTipIconFontSize, ";}}.k-alert-dialog-wrapper{overflow:hidden;padding-left:", dialog.alert.wrapperPaddingLeft, ";}.k-alert-dialog-title{line-height:", dialog.alert.tipIconLineHeight, ";font-size:", dialog.alert.titleFontSize, ";}}}");
}