import { useInstance, remove, onBeforeUnmount } from 'intact-vue-next';
import { position } from '../position';
import { onOpen, onClosed } from './fixBody';
import { SHOW } from './constants';
export function usePosition(elementRef) {
  var instance = useInstance();
  var fixedBody = false;
  instance.on(SHOW, onEnter);
  onBeforeUnmount(function () {
    if (fixedBody && instance.get('value')) {
      onClosed();
    }
  });

  function onEnter() {
    if (instance.get('overlay')) {
      fixedBody = true;
      onOpen();
    }

    center();
  }

  function center() {
    position(elementRef.value, {
      // ensure title visible always
      using: function using(feedback, position) {
        var height = feedback.element.height;
        var scrollTop = window.pageYOffset;
        var outerHeight = window.document.documentElement.clientHeight;

        if (height > outerHeight) {
          position.top = scrollTop;
        }
      },
      // let dialog padding top half of padding bottom
      my: 'center center+16%',
      at: 'center center-16%'
    });
  }

  function onAfterLeave() {
    if (fixedBody) {
      fixedBody = false;
      onClosed();
    }

    clearPosition();

    if (!instance.useAsComponent) {
      remove(instance.$vNode, document.body, false);
    }
  }

  function clearPosition() {
    var style = elementRef.value.style;
    style.left = style.top = '';
  }

  return {
    onAfterLeave: onAfterLeave
  };
}