import _extends from "@babel/runtime-corejs3/helpers/extends";
import _inheritsLoose from "@babel/runtime-corejs3/helpers/inheritsLoose";
import _concatInstanceProperty from "@babel/runtime-corejs3/core-js/instance/concat";
import { __decorate } from "tslib";
import { Component, createVNode as h, directClone, provide, inject, findDomFromVNode, createVNode, nextTick } from 'intact-vue-next';
import { bind, isTextChildren } from '../utils';
import { EMPTY_OBJ, isFunction, noop } from 'intact-shared';
import { cx } from '@emotion/css';
import { useDocumentClick, containsOrEqual } from '../../hooks/useDocumentClick';
import { Portal } from '../portal';
import { useShowHideEvents } from '../../hooks/useShowHideEvents';
import { usePosition } from './usePosition';
export var DROPDOWN = 'Dropdown';
export var ROOT_DROPDOWN = 'RootDropdown';
export var typeDefs = {
  trigger: ['hover', 'click', 'contextmenu'],
  disabled: Boolean,
  value: Boolean,
  position: [Object, 'left', 'bottom', 'right', 'top'],
  // Event is undefined in NodeJs
  of: ['self', 'parent', typeof Event === 'undefined' ? undefined : Event],
  container: [String, Function]
};
export var defaults = function defaults() {
  return {
    trigger: 'hover',
    position: {},
    of: 'self'
  };
};
export var Dropdown = /*#__PURE__*/function (_Component) {
  _inheritsLoose(Dropdown, _Component);

  function Dropdown() {
    var _context;

    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _Component.call.apply(_Component, _concatInstanceProperty(_context = [this]).call(_context, args)) || this;
    _this.menuVNode = null;
    _this.dropdown = null;
    _this.rootDropdown = null;
    _this.showedDropdown = null;
    _this.positionHook = usePosition();
    _this.timer = undefined;
    _this.triggerProps = null;
    return _this;
  }

  var _proto = Dropdown.prototype;

  _proto.init = function init() {
    var _this2 = this;

    provide(DROPDOWN, this);
    this.dropdown = inject(DROPDOWN, null);
    useDocumentClickForDropdown(this);
    useHideLastMenuOnShow(this); // is root dropdown or not

    var rootDropdown = this.rootDropdown = inject(ROOT_DROPDOWN, null);

    if (!rootDropdown) {
      provide(ROOT_DROPDOWN, this);
    }

    useShowHideEvents(); // if disabled, always set value to false

    this.watch('disabled', function (disabled) {
      if (disabled) {
        _this2.set('value', false);
      }
    });
  };

  _proto.show = function show(shouldFocus) {
    var _this3 = this;

    if (shouldFocus === void 0) {
      shouldFocus = false;
    }

    if (this.get('disabled')) return;
    clearTimeout(this.timer);
    this.set('value', true);

    if (shouldFocus) {
      nextTick(function () {
        _this3.focusFirst();
      });
    }
  };

  _proto.hide = function hide(immediately) {
    var _this4 = this;

    if (immediately === void 0) {
      immediately = false;
    }

    if (this.get('disabled')) return;
    if (!this.get('value')) return;

    if (immediately) {
      this.set('value', false);
    } else {
      this.timer = window.setTimeout(function () {
        _this4.set('value', false);
      }, 200);
    }
  };

  _proto.focusFirst = function focusFirst() {
    this.trigger('shouldFocus');
  };

  _proto.position = function position(callback) {
    if (callback === void 0) {
      callback = noop;
    }

    return this.positionHook.handle(callback);
  };

  _proto.onEnter = function onEnter(e) {
    this.callOriginalCallback(e.type === 'click' ? 'ev-click' : 'ev-mouseenter', e);
    this.show();
  };

  _proto.onContextMenu = function onContextMenu(e) {
    this.callOriginalCallback('ev-contextmenu', e);
    e.preventDefault();
    this.set('of', e);
    this.show();
  };

  _proto.onLeave = function onLeave(e) {
    this.callOriginalCallback('ev-mouseleave', e);
    this.hide();
  };

  _proto.callOriginalCallback = function callOriginalCallback(name, e) {
    var callback = this.triggerProps[name];
    if (isFunction(callback)) callback(e);
  };

  _proto.normalizeTriggerProps = function normalizeTriggerProps(props) {
    // if use kpc in react or vue, normalize props by Wrapper.props.vnode;
    if (this.$isReact || this.$isVueNext) {
      var vnode = props.vnode;
      if (!vnode) return props;
      var _props = vnode.props;
      return {
        vnode: vnode,
        'ev-click': _props.onClick,
        'ev-mouseenter': _props.onMouseEnter,
        'ev-mouseleave': _props.onMouseLeave,
        'ev-contextmenu': _props.onContextMenu,
        className: _props.className || _props.class
        /* vue-next */

      };
    } else if (this.$isVue) {// TODO:
      // return {
      // 'ev-click':  
      // }
    }

    return props;
  };

  return Dropdown;
}(Component);
Dropdown.$doubleVNodes = true;
Dropdown.typeDefs = typeDefs;
Dropdown.defaults = defaults;

Dropdown.template = function () {
  var _cx;

  var children = this.get('children');

  if (process.env.NODE_ENV !== 'production') {
    if (!Array.isArray(children) || children.length !== 2) {
      throw new Error('Dropdown must receive two children, ' + 'one is a trigger and another one is DropdownMenu');
    }
  }

  var trigger = children[0],
      menu = children[1];
  var triggerType = this.get('trigger');
  var props = {};

  if (triggerType !== 'contextmenu') {
    props['ev-click'] = this.onEnter;

    if (triggerType === 'hover') {
      props['ev-mouseenter'] = this.onEnter;
      props['ev-mouseleave'] = this.onLeave;
    }
  } else {
    props['ev-contextmenu'] = this.onContextMenu;
  }

  var clonedTrigger = isTextChildren(trigger) ? createVNode('span', null, trigger) : directClone(trigger);
  var triggerProps = this.triggerProps = this.normalizeTriggerProps(trigger.props || EMPTY_OBJ); // add a className for opening status

  var className = trigger.className || triggerProps.className;
  className = cx((_cx = {}, _cx[className] = className, _cx['k-dropdown-open'] = this.get('value'), _cx));
  clonedTrigger.props = _extends({}, triggerProps, props, {
    className: className
  });
  clonedTrigger.className = className;
  this.menuVNode = menu;
  return [clonedTrigger, // wrap the root dropdown menu to Portal to render into body
  !this.rootDropdown ? h(Portal, {
    children: menu,
    container: this.get('container')
  }) : menu];
};

__decorate([bind], Dropdown.prototype, "position", null);

__decorate([bind], Dropdown.prototype, "onEnter", null);

__decorate([bind], Dropdown.prototype, "onContextMenu", null);

__decorate([bind], Dropdown.prototype, "onLeave", null);

function useDocumentClickForDropdown(dropdown) {
  var elementRef = function elementRef() {
    return findDomFromVNode(dropdown.menuVNode, true);
  };

  var _useDocumentClick = useDocumentClick(elementRef, function (e) {
    // if click an trigger and the trigger type is hover, ignore it
    if (dropdown.get('trigger') === 'hover') {
      var triggerDom = findDomFromVNode(dropdown.$lastInput, true);
      var target = e.target;
      if (containsOrEqual(triggerDom, target)) return;
    }

    dropdown.hide(true);
  }, true),
      addDocumentClick = _useDocumentClick[0],
      removeDocumentClick = _useDocumentClick[1];

  dropdown.on('show', addDocumentClick);
  dropdown.on('hide', removeDocumentClick);
}

function useHideLastMenuOnShow(dropdown) {
  var parentDropdown = dropdown.dropdown;
  dropdown.on('show', function () {
    if (parentDropdown) {
      var showed = parentDropdown.showedDropdown;

      if (showed && showed !== dropdown) {
        showed.hide(true);
      }

      parentDropdown.showedDropdown = dropdown;
    }
  });
}