import _mapInstanceProperty from "@babel/runtime-corejs3/core-js/instance/map";
import { css } from '@emotion/css';
import { theme } from '../../styles/theme';
import { deepDefaults, sizes } from '../../styles/utils';
import '../../styles/global';

var _deepDefaults = deepDefaults(theme, {
  input: deepDefaults({
    get fontSize() {
      return theme.default.fontSize;
    },

    get height() {
      return theme.default.height;
    },

    width: '300px',

    get border() {
      return "1px solid " + theme.color.border;
    },

    get paddingGap() {
      return theme.default.padding;
    },

    bgColor: '#fff',

    get color() {
      return theme.color.text;
    },

    get focusBorder() {
      return "1px solid " + theme.color.primary;
    },

    get hoverBorder() {
      return "1px solid " + theme.color.darkBorder;
    },

    get borderRadius() {
      return theme.borderRadius;
    },

    get placeholderColor() {
      return theme.color.placeholder;
    },

    // clearable
    get clearIconColor() {
      return theme.color.placeholder;
    },

    clearIconGap: '3px',

    // textarea
    get textareaPadding() {
      return "5px " + input.paddingGap;
    },

    // group
    get groupBgColor() {
      return theme.color.bg;
    },

    groupPaddingGap: '16px',

    // disabled
    get disabledBorderColor() {
      return theme.color.disabledBorder;
    },

    get disabledColor() {
      return theme.color.disabled;
    },

    get disabledBgColor() {
      return theme.color.disabledBg;
    },

    // search input
    search: {
      suffixMarginRight: '15px'
    }
  }, sizes.reduce(function (memo, size) {
    var styles = theme[size];
    memo[size] = {
      get fontSize() {
        return styles.fontSize;
      },

      get height() {
        return styles.height;
      },

      get paddingGap() {
        return styles.padding;
      }

    };

    if (size === 'large') {
      // use default padding for large size
      Object.defineProperty(memo.large, 'paddingGap', {
        get: function get() {
          return theme.default.padding;
        }
      });
    }

    return memo;
  }, {}))
}),
    input = _deepDefaults.input;

export function makeStyles() {
  return /*#__PURE__*/css("display:inline-block;width:", input.width, ";vertical-align:middle;.k-input-wrapper{display:inline-block;width:100%;position:relative;}.k-input-inner{display:inline-block;width:100%;border:", input.border, ";background-color:", input.bgColor, ";transition:border ", theme.transition, ",background ", theme.transition, ",box-shadow ", theme.transition, ";border-radius:", input.borderRadius, ";outline:none;position:relative;color:", input.color, ";&:hover{border:", input.hoverBorder, ";z-index:1;}&:focus{border:", input.focusBorder, ";z-index:1;}&::placeholder{color:", input.placeholderColor, ";}}&.k-fluid{width:100%;}.k-input-prefix,.k-input-suffix{position:absolute;top:50%;transform:translateY(-50%);z-index:2;}.k-input-clear{display:inline-block;vertical-align:top;cursor:pointer;opacity:0;transition:opacity ", theme.transition, ";pointer-events:none;color:", input.clearIconColor, ";&:hover{color:", theme.color.primary, ";}+*{margin-left:", input.clearIconGap, ";}}&:hover .k-input-clear.k-input-show{opacity:1;pointer-events:all;}&.k-stack-clear{.k-input-clear{position:absolute;z-index:1;right:0;&.k-input-show+i{transition:opacity ", theme.transition, ";}}&:hover{.k-input-clear.k-input-show+i{opacity:0;}}}&.k-group{display:table;.k-input-inner{border-radius:0;}.k-input-wrapper:first-child{.k-input-inner{border-radius:", input.borderRadius, " 0 0 ", input.borderRadius, ";}}.k-input-wrapper:last-child{.k-input-inner{border-radius:0 ", input.borderRadius, " ", input.borderRadius, " 0;}}}.k-input-prepend,.k-input-append{display:table-cell;width:1px;vertical-align:middle;background-color:", input.groupBgColor, ";border:", input.border, ";text-align:center;white-space:nowrap;.k-btn{margin:-1px 0;border-radius:0;border:none;}.k-select{margin:-1px;text-align:left;}}.k-input-prepend{border-radius:", input.borderRadius, " 0 0 ", input.borderRadius, ";.k-select{z-index:1;.k-select-wrapper{border-radius:", theme.borderRadius, " 0 0 ", theme.borderRadius, ";}}}.k-input-append{border-radius:0 ", input.borderRadius, " ", input.borderRadius, " 0;.k-select{.k-select-wrapper{border-radius:0 ", theme.borderRadius, " ", theme.borderRadius, " 0;}}}.k-input-padding{padding:0 ", input.groupPaddingGap, ";}.k-input-prepend{border-right:none;}.k-input-append{border-left:none;}&.k-disabled{color:", input.disabledColor, ";cursor:not-allowed;.k-input-inner{color:", input.disabledColor, ";border-color:", input.disabledBorderColor, ";background:", input.disabledBgColor, ";cursor:not-allowed;}}", _mapInstanceProperty(sizes).call(sizes, function (size) {
    var styles = input[size];
    var sizeClassName = /*#__PURE__*/css("font-size:", styles.fontSize, ";.k-input-inner{height:", styles.height, ";line-height:", styles.height, ";font-size:", styles.fontSize, ";padding:0 ", styles.paddingGap, ";}.k-input-prefix{left:", styles.paddingGap, ";}.k-input-suffix{right:", styles.paddingGap, ";}&.k-with-prefix{.k-input-inner{padding-left:calc(", styles.paddingGap, " + 1rem + ", input.clearIconGap, ");}}&.k-with-suffix,&.k-clearable{.k-input-inner{padding-right:calc(", styles.paddingGap, " + 1rem + ", input.clearIconGap, ");}}&:not(.k-stack-clear).k-with-suffix.k-clearable{.k-input-inner{padding-right:calc(", styles.paddingGap, " + 2rem + ", input.clearIconGap, " * 2);}}");
    if (size === 'default') return sizeClassName;
    return /*#__PURE__*/css("&.k-", size, "{", sizeClassName, ";}");
  }), ".k-textarea{padding:", input.textareaPadding, ";height:auto;line-height:1.5;vertical-align:top;}.k-input-fake{position:absolute;visibility:hidden;top:0;white-space:nowrap;}&.k-auto-width{width:auto;.k-input-inner{box-sizing:content-box;}}");
}
export function makeSearchStyles() {
  return /*#__PURE__*/css("position:relative;display:inline-block;.k-input{transition:width ", theme.transition, ";}.k-btn{position:absolute;top:0;right:0;z-index:1;}.k-input-suffix{margin-right:", input.search.suffixMarginRight, ";}&.k-hide{", _mapInstanceProperty(sizes).call(sizes, function (size) {
    if (size == 'default') return;
    return /*#__PURE__*/css(".k-input.k-", size, "{width:", input[size].height, ";}");
  }), ";.k-input{width:", input.default.height, ";}.k-input-inner{padding:0!important;}}&.k-line{.k-input-inner{border-width:0;}&.k-open{.k-input-inner{border-bottom-width:1px;}}}");
}