export declare type Options = {
    of?: Window | HTMLElement | Document | MouseEvent;
    at?: string | [string, string];
    my?: string | [string, string];
    collision?: Collision | [Collision, Collision];
    within?: WithinInfo['element'];
    collisionDirection?: ['left'] | ['top'] | ['left', 'top'];
    using?: (feedback: Feedback, position: Offset) => void;
};
declare type Collision = keyof Rules | 'none';
declare type WithinInfo = {
    element: HTMLElement | Window | Document;
    isWindow: boolean;
    isDocument: boolean;
    offset: Offset;
    width: number;
    height: number;
    scrollLeft: number;
    scrollTop: number;
};
declare type Offset = {
    top: number;
    left: number;
};
declare type Horizontal = 'left' | 'center' | 'right';
declare type Vertical = 'top' | 'center' | 'bottom';
declare type CollisionPosition = {
    marginTop: number;
    marginLeft: number;
};
declare type Rules = {
    fit: Rule;
    flip: Rule;
    flipfit: Rule;
};
declare type Rule = {
    left: (position: Offset, data: RuleData) => void;
    top: (position: Offset, data: RuleData) => void;
};
declare type RuleData = {
    targetWidth: number;
    targetHeight: number;
    elemWidth: number;
    elemHeight: number;
    collisionPosition: CollisionPosition;
    collisionWidth: number;
    collisionHeight: number;
    offset: [number, number];
    my: [string, string];
    at: [string, string];
    within: WithinInfo;
    elem: Options['of'];
};
export declare type Feedback = {
    target: FeedbackElement;
    element: FeedbackElement;
    horizontal: Horizontal;
    vertical: Vertical;
    important: 'vertical' | 'horizontal';
};
export declare type FeedbackElement = {
    element: Options['of'];
    left: number;
    top: number;
    width: number;
    height: number;
};
export declare function getDocumentOrWindowRect(elem: Document | Window): {
    width: number;
    height: number;
};
export declare function scrollbarWidth(): number;
export default function position(elem: HTMLElement, options: Options): void;
export { position };
