import { Component, TypeDefs } from 'intact-vue-next';
import { TableMerge } from './useMerge';
import { TooltipProps } from '../tooltip/tooltip';
export interface TableProps<T = any> {
    data?: T[];
    fixHeader?: boolean | string | number;
    stickHeader?: boolean | string | number;
    checkType?: 'checkbox' | 'radio' | 'none';
    checkedKeys?: TableRowKey[];
    rowKey?: (value: T, index: number) => TableRowKey;
    rowCheckable?: boolean;
    disableRow?: (value: T, index: number, key: TableRowKey) => boolean;
    type?: 'default' | 'border' | 'grid';
    stripe?: boolean;
    rowClassName?: (value: T, index: number, key: TableRowKey) => string | undefined;
    group?: Record<string, any>;
    sort?: TableSortValue;
    loading?: boolean;
    merge?: TableMerge;
    expandedKeys?: TableRowKey[];
    rowExpandable?: boolean;
    selectedKeys?: TableRowKey[];
    rowSelectable?: boolean | 'single' | 'multiple';
    childrenKey?: string;
    indent?: number;
    spreadKeys?: TableRowKey[];
    tooltipPosition?: TooltipProps['position'];
    tooltipContainer?: TooltipProps['container'];
    keepStatus?: boolean;
    showIndeterminate?: boolean;
    resizable?: boolean;
    minColWidth?: number;
    widthStoreKey?: string;
}
export interface TableEvents<T = any> {
    clickRow: [T, number, TableRowKey];
    dragstart: [{
        key: TableRowKey;
        from: number;
    }];
    dragend: [{
        key: TableRowKey;
        from: number;
        to: number;
    }];
}
export interface TableBlocks<T = unknown> {
    empty: null;
    tooltip: [T, number];
    expand: [T, number];
}
export declare type TableRowKey = string | number;
export declare type TableSortValue = {
    key?: string;
    type?: 'desc' | 'asc';
};
export declare class Table<T = any> extends Component<TableProps<T>, TableEvents<T>> {
    static template: import("intact").Template<any>;
    static typeDefs: Required<TypeDefs<TableProps<unknown>>>;
    static defaults: () => Partial<TableProps<any>>;
    private tree;
    private columns;
    private stickyHeader;
    private resizable;
    private fixedColumns;
    private disableRow;
    private merge;
    private checked;
    private sortable;
    private expandable;
    private selected;
    private resetRowStatus;
    private draggable;
    getCheckedData(): any[];
    getSelectedData(): any[];
    scrollToRowByIndex(index: number): Promise<void>;
    scrollToRowByKey(key: TableRowKey): Promise<void>;
    exportTable(data?: any[] | undefined, filename?: string): Promise<void>;
    private getData;
    private clickRow;
    private onChangeGroup;
}
