import { useInstance, onMounted, onUnmounted } from 'intact-vue-next';
import { useState } from '../../hooks/useState';
import { cx } from '@emotion/css';
import { isNullOrUndefined, isString, error } from 'intact-shared';
import { throttle } from '../utils';
import { watchState } from '../../hooks/useState';
import { createContext } from '../context';
export var context = createContext();
export function useFixedColumns(getColumns, scrollRef, widthMap) {
  var instance = useInstance();
  var scrollPosition = useState(null);
  var hasFixed = useState(false);
  var scrollLeft = 0;
  var hasFixedLeft = false;
  var hasFixedRight = false;
  var offsetMap = {};

  function handleFixedColumns() {
    var columns = getColumns();
    hasFixedLeft = false;
    hasFixedRight = false;
    offsetMap = {};
    columns.forEach(function (columns, row) {
      var offset = 0;
      columns.forEach(function (vNode, col) {
        var props = vNode.props;
        var key = props.key;
        offsetMap[key] = 0;
        var fixed = props.fixed;

        if (fixed === 'left') {
          var value = 0;
          var prevVNode = props.prevVNode;

          while (prevVNode) {
            var _props = prevVNode.props;

            if (_props.fixed === 'left') {
              if (process.env.NODE_ENV !== 'production') {
                validateWidth(_props);
              }

              value += widthMap.value[_props.key] || parseInt(_props.width, 10) || 0;
            }

            prevVNode = _props.prevVNode;
          }

          offsetMap[key] = value;
          hasFixedLeft = true;
        } else if (fixed === 'right') {
          var _value = 0;
          var lastVNode = columns[columns.length - 1];

          while (vNode !== lastVNode) {
            var _props2 = lastVNode.props;

            if (_props2.fixed === 'right') {
              if (process.env.NODE_ENV !== 'production') {
                validateWidth(_props2);
              }

              _value += widthMap.value[_props2.key] || parseInt(_props2.width, 10) || 0;
            }

            lastVNode = _props2.prevVNode;
          }

          offsetMap[key] = _value;
          hasFixedRight = true;
        }
      });
    });
    hasFixed.set(hasFixedLeft || hasFixedRight);
  }

  function onScroll(e) {
    if (!hasFixed.value) return;
    var scrollDom = e.target;
    var newScrollLeft = scrollDom.scrollLeft;

    if (scrollLeft !== newScrollLeft) {
      scrollLeft = newScrollLeft;
      var maxScroll = scrollDom.scrollWidth - scrollDom.offsetWidth;
      scrollPosition.set(scrollLeft === 0 ? 'left' : scrollLeft >= maxScroll ? 'right' : 'middle');
    }
  }

  function updateScrollPositionOnResize() {
    var scrollDom = scrollRef.value;

    if (scrollDom.scrollWidth - scrollDom.offsetWidth <= 0) {
      scrollPosition.set(null);
    } else {
      scrollPosition.set('left');
    }
  }

  instance.on('$receive:children', handleFixedColumns);
  watchState(widthMap, handleFixedColumns);
  var throttleUpdate = throttle(function () {
    if (instance.$unmounted) return;
    updateScrollPositionOnResize();
  }, 100);
  onMounted(function () {
    updateScrollPositionOnResize();
    window.addEventListener('resize', throttleUpdate);
  });
  onUnmounted(function () {
    window.removeEventListener('resize', throttleUpdate);
  });
  return {
    scrollPosition: scrollPosition,
    onScroll: onScroll,
    hasFixed: hasFixed,
    getHasFixedLeft: function getHasFixedLeft() {
      return hasFixedLeft;
    },
    getOffsetMap: function getOffsetMap() {
      return offsetMap;
    }
  };
}
export function getClassAndStyleForFixed(_ref, offset, checkType) {
  var _cx, _ref2;

  var className = _ref.className,
      fixed = _ref.fixed;
  var extraOffset = checkType && checkType !== 'none' && fixed === 'left' ? 40 : 0;
  return {
    className: cx((_cx = {}, _cx[className] = !!className, _cx["k-fixed-" + fixed] = !!fixed, _cx)),
    style: fixed ? (_ref2 = {}, _ref2[fixed] = offset + extraOffset + "px", _ref2) : null
  };
}

function validateWidth(_ref3) {
  var width = _ref3.width,
      key = _ref3.key;

  if (isNullOrUndefined(width) || isString(width) && !/\d+(px)?/.test(width)) {
    error("The width must be specified with 'px' when the column has fixed. <TableColumn key=\"" + key + "\" />");
  }
}