import _extends from "@babel/runtime-corejs3/helpers/extends";
import _mapInstanceProperty from "@babel/runtime-corejs3/core-js/instance/map";
import _JSON$stringify from "@babel/runtime-corejs3/core-js/json/stringify";
import { useInstance, findDomFromVNode, createRef } from 'intact-vue-next';
import { createContext } from '../context';
import { useDraggable } from '../../hooks/useDraggable';
import { useState } from '../../hooks/useState';
export var context = createContext();
var hasLocalStorage = typeof localStorage !== 'undefined';
export function useResizable(scrollRef) {
  var instance = useInstance();
  var tableRef = createRef();
  var widthMap = useState({});
  var tableWidth = useState(null); // if exist widthStoreKey, we get the default width from localStorage

  var _instance$get = instance.get(),
      widthStoreKey = _instance$get.widthStoreKey;

  if (widthStoreKey && hasLocalStorage) {
    var data = localStorage.getItem(widthStoreKey);

    if (data) {
      try {
        var _JSON$parse = JSON.parse(data),
            map = _mapInstanceProperty(_JSON$parse),
            width = _JSON$parse.width;

        if (map) {
          widthMap.set(map);
        }

        if (width) {
          tableWidth.set(width);
        }
      } catch (e) {}
    }
  }

  var containerWidth;
  var x;
  var prevVNode;
  var prevTh;
  var currentVNode;
  var currentTh;
  var minWidth;

  function onStart(e) {
    containerWidth = scrollRef.value.clientWidth;
    x = e.clientX; // find the previous column but exclude the hidden columns, #467

    prevVNode = currentVNode = e._vNode;
    prevTh;

    do {
      prevVNode = prevVNode.props.prevVNode;
      prevTh = findDomFromVNode(prevVNode, true);
    } while (window.getComputedStyle(prevTh).display === 'none');

    currentTh = findDomFromVNode(currentVNode, true);
    minWidth = prevVNode.props.minWidth || instance.get('minColWidth');
  }

  function onMove(e) {
    var _map;

    var delX = e.clientX - x;
    if (delX === 0) return;
    var newPrevWidth = prevTh.offsetWidth + delX;
    if (newPrevWidth < minWidth && delX < 0) return;
    var newCurrentWidth = currentTh.offsetWidth - delX;
    var newTableWidth = tableRef.value.offsetWidth + delX;
    var currentKey = currentVNode.props.key;
    var prevKey = prevVNode.props.key;
    x = e.clientX;
    var map = (_map = {}, _map[prevKey] = newPrevWidth, _map);

    if (containerWidth > newTableWidth) {
      map[currentKey] = newCurrentWidth;
    } else {
      tableWidth.set(newTableWidth);
    }

    widthMap.set(_extends({}, widthMap.value, map));
  }

  function onEnd() {
    // store width if has widthStoreKey
    var _instance$get2 = instance.get(),
        widthStoreKey = _instance$get2.widthStoreKey;

    if (widthStoreKey) {
      localStorage.setItem(widthStoreKey, _JSON$stringify({
        map: widthMap.value,
        width: tableWidth.value
      }));
    }
  }

  function getWidth(key) {
    var width = widthMap.value[key]; // if (width === 'auto') return width;

    if (width) return width + "px";
    return null;
  }

  return _extends({}, useDraggable({
    onStart: onStart,
    onMove: onMove,
    onEnd: onEnd
  }), {
    getWidth: getWidth,
    tableRef: tableRef,
    tableWidth: tableWidth,
    widthMap: widthMap
  });
}