import { createRef, useInstance, onMounted, onBeforeUnmount } from 'intact-vue-next';
import { useState } from '../../hooks/useState';
import { isStringOrNumber, isNull } from 'intact-shared';
export function useStickyHeader() {
  var instance = useInstance();
  var elementRef = createRef();
  var scrollRef = createRef();
  var headRef = createRef();
  var stickHeader = useState(null);

  function excludeStickHeader(_ref) {
    var offsetTop = _ref.offsetTop;

    var _elementRef$value$get = elementRef.value.getBoundingClientRect(),
        bottom = _elementRef$value$get.bottom;

    return bottom <= offsetTop;
  } // when the scroll element scroll horizontally, scroll the sticky header too 


  function onScroll() {
    if (isNull(stickHeader.value)) return;
    var scrollLeft = scrollRef.value.scrollLeft;
    var affixHeadWrapper = headRef.value.parentElement;
    affixHeadWrapper.scrollLeft = scrollLeft;
  }

  instance.on('$receive:stickHeader', function (v) {
    stickHeader.set(v === true ? 0 : isStringOrNumber(v) ? +v : null);
  });
  onMounted(function () {
    scrollRef.value.addEventListener('scroll', onScroll);
  });
  onBeforeUnmount(function () {
    scrollRef.value.removeEventListener('scroll', onScroll);
  });
  return {
    stickHeader: stickHeader,
    excludeStickHeader: excludeStickHeader,
    elementRef: elementRef,
    scrollRef: scrollRef,
    headRef: headRef
  };
}