import { useInstance, createRef, onMounted, onUpdated } from 'intact-vue-next';
import { useState } from '../../hooks/useState';
import { useResizeObserver } from '../../hooks/useResizeObserver';
export function useScroll() {
  var instance = useInstance();
  var scrollRef = createRef();
  var wrapperRef = createRef();
  var isScroll = useState(false);
  var left = useState(0);
  var enablePrev = useState(false);
  var enableNext = useState(false);
  onMounted(refresh);
  onUpdated(refresh);
  useResizeObserver(scrollRef, refresh);
  instance.watch('value', scrollActiveToView, {
    presented: true
  });

  function refresh() {
    var _instance$get = instance.get(),
        vertical = _instance$get.vertical;

    if (vertical) return;
    var scrollWidth = scrollRef.value.offsetWidth;
    var wrapperWidth = wrapperRef.value.offsetWidth;
    isScroll.set(scrollWidth < wrapperWidth);
    updateTransition(left.value, scrollWidth, wrapperWidth);
  }

  function updateTransition(_left, scrollWidth, wrapperWidth) {
    if (_left === void 0) {
      _left = left.value;
    }

    if (scrollWidth === void 0) {
      scrollWidth = scrollRef.value.offsetWidth;
    }

    if (wrapperWidth === void 0) {
      wrapperWidth = wrapperRef.value.offsetWidth;
    }

    var prev = true;
    var next = true;

    if (_left >= 0) {
      _left = 0;
      prev = false;
    } else if (_left <= scrollWidth - wrapperWidth) {
      _left = scrollWidth - wrapperWidth;
      next = false;
    }

    left.set(_left);
    enablePrev.set(prev);
    enableNext.set(next);
  }

  function prev() {
    var scrollWidth = scrollRef.value.offsetWidth;
    updateTransition(left.value + scrollWidth);
  }

  function next() {
    var scrollWidth = scrollRef.value.offsetWidth;
    updateTransition(left.value - scrollWidth);
  }

  function scrollActiveToView() {
    var _instance$get2 = instance.get(),
        vertical = _instance$get2.vertical;

    if (vertical) return;
    var activeTab = scrollRef.value.querySelector('.k-tab.k-active');

    if (activeTab) {
      var _left = left.value; // don't use getBoundingClientRect, it has transform and will return digital number

      var activeRect = {
        left: activeTab.offsetLeft,
        width: activeTab.offsetWidth
      };
      var scrollRect = {
        width: scrollRef.value.offsetWidth
      };
      var wrapperRect = {
        left: wrapperRef.value.offsetLeft,
        width: wrapperRef.value.offsetWidth
      };
      var x = activeRect.left - wrapperRect.left;

      if (_left + x <= 0) {
        _left = -x;
      } else if (x + activeRect.width + _left > scrollRect.width) {
        _left = -x - activeRect.width + scrollRect.width;
      }

      left.set(_left);
    } else {
      left.set(0);
    }
  }

  return {
    isScroll: isScroll,
    left: left,
    enablePrev: enablePrev,
    enableNext: enableNext,
    prev: prev,
    next: next,
    scrollRef: scrollRef,
    wrapperRef: wrapperRef
  };
}