import _mapInstanceProperty from "@babel/runtime-corejs3/core-js/instance/map";
import { css } from '@emotion/css';
import { deepDefaults, sizes, palette } from '../../styles/utils';
import { theme } from '../../styles/theme';
import '../../styles/global';
var types = ['primary', 'warning', 'danger', 'success', 'disabled'];

var _deepDefaults = deepDefaults(theme, {
  tag: deepDefaults({
    get borderColor() {
      return theme.color.border;
    },

    get borderRadius() {
      return theme.borderRadius;
    },

    get fontSize() {
      return theme.default.fontSize;
    },

    padding: "0 8px",
    height: '20px',
    close: {
      fontSize: '20px',
      gap: '8px'
    },
    disabled: {
      get color() {
        return theme.color.disabled;
      },

      get borderColor() {
        return theme.color.disabledBorder;
      },

      get bgColor() {
        return theme.color.disabledBg;
      }

    },
    large: {
      padding: "0px 16px",
      height: '32px',
      close: {
        fontSize: '24px',
        gap: '12px'
      }
    },
    small: {
      padding: "0 4px",
      height: '18px',
      close: {
        fontSize: '18px',
        gap: '4px'
      }
    },
    mini: {
      padding: "0 1px",
      height: '14px',
      close: {
        fontSize: '16px',
        gap: '2px'
      }
    }
  }, types.reduce(function (memo, type) {
    if (type === 'disabled') return memo;
    memo[type] = {
      get color() {
        return theme.color[type];
      },

      get bgColor() {
        return palette(theme.color[type], -4);
      }

    };
    return memo;
  }, {}))
}),
    tag = _deepDefaults.tag;

export function makeStyles() {
  return /*#__PURE__*/css("display:inline-flex;align-items:center;padding:", tag.padding, ";border:1px solid ", tag.borderColor, ";border-radius:", tag.borderRadius, ";font-size:", tag.fontSize, ";height:", tag.height, ";.k-tag-close{font-size:", tag.close.fontSize, ";margin-left:", tag.close.gap, ";}", _mapInstanceProperty(types).call(types, function (t) {
    var styles = tag[t];
    return /*#__PURE__*/css("&.k-", t, "{color:", styles.color, ";border-color:", styles.color, ";background:", styles.bgColor, ";}");
  }), " ", _mapInstanceProperty(sizes).call(sizes, function (s) {
    if (s === 'default') return;
    var styles = tag[s];
    return /*#__PURE__*/css("&.k-", s, "{padding:", styles.padding, ";height:", styles.height, ";.k-tag-close{font-size:", styles.close.fontSize, ";margin-left:", styles.close.gap, ";}}");
  }), "&.k-dashed{border-style:dashed;}&.k-none{border:none;}");
}